/****************************************************************************
              Copyright (c) 2022 Maxima Technologies and Systems
                           All Rights Reserved

   File Name          : memory_management.c
   Author             : Rajesh
   Version            : V0.0.0.1
   Origination Date   : 21/01/2022
   Description        : C Source File.
   Environment        : Independent
   Processor:         : STM32H743IIT6

-----------------------------------------------------------------------------
   Description:

   This file contains the memory management application

-----------------------------------------------------------------------------
	File Revision History:
-----------------------------------------------------------------------------
   Version             Date                 Programmer
   0.0.0.1             21/01/2022           Rajesh

   Notes:              - Initial Version
-----------------------------------------------------------------------------
*****************************************************************************
THIS FILE CONTAINS INTELLECTUAL PROPERTY OF MAXIMA TECHNOLOGIES AND SYSTEMS.
UNAUTHORIZED REDISTRIBUTION OF THIS SOURCE CODE, IN WHOLE OR PART, WITHOUT
THE EXPRESS WRITTEN PERMISSION OF MAXIMA TECHNOLOGIES IS STRICTLY PROHIBITED.
****************************************************************************/

/* Include Header files */
#include "../Inc/Common.h"

/* USER CODE BEGIN PV */
extern pvt_variable pv;
static StaticQueue_t xStaticQueue;
uint8_t ucQueueStorageArea[ ((MEM_QBUFFSIZE/MEM_QPKTSIZE) + 1) * sizeof(uint32_t*)];
/* USER CODE END PV */

/** @brief Memory Management Buffer initialization
 *
 *  This function takes the fixed memory size from the static memory.
 *  Queue the pointer into the OS Queue. Later, if the task needs any
 *  memory, it can take it from this queue and use this memory.
 *  If this memory is not required further, it can queue this back into
 *  the OS memory Queue
 *
 *  @param  None
 *
 *  @return None
 *
 */
void start_memory_management(void)
{
	BaseType_t rc;
	uint32_t*  bufptr;

	/* Create Memory Management Queue */
	pv.membufq = xQueueCreateStatic( (MEM_QBUFFSIZE/MEM_QPKTSIZE),
			sizeof(uint32_t*),ucQueueStorageArea,&xStaticQueue);
	if(0 == pv.membufq)
	{
		Error_Handler();
	}

	/* Queue the fix memory pointer into the Memory Management Queue */
	for(uint16_t index=0; index < MEM_QBUFFSIZE/MEM_QPKTSIZE; index++ )
	{
		bufptr = (uint32_t* )&pv.membuf[index* MEM_QPKTSIZE];
		while(1)
		{
			rc = xQueueSend( pv.membufq,
					( void * ) &bufptr,
					( TickType_t ) MEM_Q_TIMEOUT );

			if (pdTRUE == rc)
				break;
		}
	}
}


/** @brief Get the Memory address from the Management Queue
 *
 *  This function returns the predefined memory address from the
 *  the Memory Management Queue
 *
 *  @param  None
 *
 *  @return allocated address
 *
 */
uint8_t *ai280_memory_alloc()
{
	uint8_t *mbuf = NULL;
	int8_t res = 0;

    do
	{
		/* Get the memory from the Memory Management Q */
		xQueueReceive( pv.membufq, &mbuf , ( TickType_t ) MEM_Q_TIMEOUT );
		if (NULL != mbuf)
			break;

		res++;

	}while(MAX_RETRY_Q > res);

	if(MAX_RETRY_Q == res)
		return NULL;

	return mbuf;
}
/** @brief Put back the Memory address to the Management Queue
 *
 *  This function puts the predefined memory address to the
 *  the Memory Management Queue
 *
 *  @param  memory address
 *
 *  @return none
 *
 */
void ai280_memory_free(uint8_t *mbuf)
{
	/* Put the address to the Memory Management Q */
   	xQueueSend( pv.membufq, &mbuf , ( TickType_t ) MEM_Q_TIMEOUT );
}

#if (SDK_SERVICE_FDCAN == PS_ENABLE)

#define FDCAN_PKT_SIZE	12

/** @brief CAN1Rx Memory Management Buffer initialization
 *
 *  This function takes the fixed memory size from the static memory.
 *  Queue the pointer into the OS Queue. Later, if the task needs any
 *  memory, it can take it from this queue and use this memory.
 *  If this memory is not required further, it can queue this back into
 *  the OS memory Queue
 *
 *  @param  None
 *
 *  @return None
 *
 */
#define MEM_CANQBUFFSIZE 120
static StaticQueue_t xStaticCAN1Queue;
static StaticQueue_t xStaticCAN1RxQueue;
uint8_t ucCAN1QueueStorageArea[((MEM_CANQBUFFSIZE/FDCAN_PKT_SIZE) + 1) * sizeof(uint32_t*)];
uint8_t ucCAN1RxQueueStorageArea[((MEM_CANQBUFFSIZE/FDCAN_PKT_SIZE) + 1) * sizeof(uint32_t*)];

void can1_rx_memory_management(void)
{
	BaseType_t rc;
	uint32_t*  bufptr;

	/* Create Memory Management Queue for the CAN1 Buffer Rx */
	pv.memcan1bufq = xQueueCreateStatic( (MEM_CANQBUFFSIZE/FDCAN_PKT_SIZE),
			sizeof(uint32_t*),ucCAN1QueueStorageArea,&xStaticCAN1Queue);
	if(0 == pv.memcan1bufq)
	{
		Error_Handler();
	}

	/* Queue the fix memory pointer into the Memory Management Queue */
	for(uint16_t index=0; index < MEM_CANQBUFFSIZE/FDCAN_PKT_SIZE; index++ )
	{
		bufptr = (uint32_t* )&pv.can1membuf[index* FDCAN_PKT_SIZE];
		while(1)
		{
			rc = xQueueSend( pv.memcan1bufq,
					( void * ) &bufptr,
					( TickType_t ) MEM_Q_TIMEOUT );

			if (pdTRUE == rc)
				break;
		}
	}

	/* Create Memory Management Queue for the CAN1 Buffer Rx */
	pv.memcan1rxbufq = xQueueCreateStatic( (MEM_CANQBUFFSIZE/FDCAN_PKT_SIZE),
			sizeof(uint32_t*),ucCAN1RxQueueStorageArea,&xStaticCAN1RxQueue);
	if(0 == pv.memcan1rxbufq)
	{
		Error_Handler();
	}
}


/** @brief Get the Memory address from the CAN1Rx Management Queue
 *
 *  This function returns the predefined memory address from the
 *  the Memory Management Queue
 *
 *  @param  None
 *
 *  @return allocated address
 *
 */
uint8_t *can1rx__memory_alloc()
{
	uint8_t *mbuf = NULL;
	int8_t res = 0;

    do
	{
		/* Get the memory from the Memory Management Q */
		xQueueReceive( pv.memcan1bufq, &mbuf , ( TickType_t ) MEM_Q_TIMEOUT );
		if (NULL != mbuf)
			break;

		res++;

	}while(MAX_RETRY_Q > res);

	if(MAX_RETRY_Q == res)
		return NULL;

	return mbuf;
}

/** @brief Put back the Memory address to the CAN1 Rx Management Queue
 *
 *  This function puts the predefined memory address to the
 *  the Memory Management Queue
 *
 *  @param  memory address
 *
 *  @return none
 *
 */
void canrx1_memory_free(uint8_t *mbuf)
{
	/* Put the address to the Memory Management Q */
   	xQueueSend( pv.memcan1bufq, &mbuf , ( TickType_t ) MEM_Q_TIMEOUT );
}

uint16_t get_can1_availablerxdata()
{
	uint16_t res;
	uint16_t total_size = (MEM_CANQBUFFSIZE/FDCAN_PKT_SIZE);

	res = uxQueueSpacesAvailable(pv.memcan1rxbufq);

	res = total_size - res;

    return res;
}

void clear_can1_rxQ()
{
	uint8_t  *mbuf = NULL;
    uint8_t  index;
	uint16_t itr = ((MEM_CANQBUFFSIZE/FDCAN_PKT_SIZE) + 1);

	for(index = 0; index < itr ; index++)
	{
		/* Get the memory from the Memory Management Q */
		xQueueReceive( pv.memcan1rxbufq, &mbuf , ( TickType_t ) MEM_Q_TIMEOUT );

		if (NULL != mbuf)
			canrx1_memory_free(mbuf);

		mbuf = NULL;
	}
}

void put_can1_data_to_rxbufq(uint8_t *rx_data)
{
	uint8_t res;

	/* send the data to rx buffer Q */
	xQueueSend( pv.memcan1rxbufq, &rx_data , ( TickType_t ) MEM_Q_TIMEOUT );

	/* get the available data */
	res = get_can1_availablerxdata();

	if(FDCAN1_ID == FDCAN_STANDARD_ID)
	{
		/* update the packet size */
		res = (res * 8);
	}
	else
	{
		/* update the packet size */
		res = (res * 12);
	}

	Set_DL(CAN1_RX_DATA_SIZE, &res);

	if(0 == res)
	{
		/* data buffer is empty */
		Set_DL(CAN1_RX_DATA_IS_AVAILABLE, &res);
	}
	else
	{
		res = TRUE;
		/* data is available in the buffer */
		Set_DL(CAN1_RX_DATA_IS_AVAILABLE, &res);
	}
}

uint8_t *get_can1_data_from_rxbufq(uint8_t *user_buf)
{
	uint8_t *mbuf = NULL;
	int8_t res = 0;

    do
	{
		/* Get the memory from the Memory Management Q */
		xQueueReceive( pv.memcan1rxbufq, &mbuf , ( TickType_t ) MEM_Q_TIMEOUT );
		if (NULL != mbuf)
			break;

		res++;

	}while(MAX_RETRY_Q > res);

	if(MAX_RETRY_Q == res)
		return NULL;

	memcpy(user_buf, mbuf, FDCAN_PKT_SIZE);

	canrx1_memory_free(mbuf);

	/* get the available data */
	res = get_can1_availablerxdata();

	if(FDCAN1_ID == FDCAN_STANDARD_ID)
	{
		/* update the packet size */
		res = (res * 8);
	}
	else
	{
		/* update the packet size */
		res = (res * 12);
	}

	Set_DL(CAN1_RX_DATA_SIZE, (uint8_t *)&res);

	if(0 == res)
	{
		/* data buffer is empty */
		Set_DL(CAN1_RX_DATA_IS_AVAILABLE, (uint8_t *)&res);
	}
	else
	{
		res = TRUE;
		/* data is available in the buffer */
		Set_DL(CAN1_RX_DATA_IS_AVAILABLE, (uint8_t *)&res);
	}

	return mbuf;
}

#endif	//SDK_SERVICE_FDCAN

/**************** (C) COPYRIGHT 2022 Maxima Technologies *******************/
/***************************** END OF FILE *********************************/
